%%%%%%%%%%%%%%%%%%%%%
% Mar 10, 2017
% Add Weight based on self and neighbor
%%%%%%%%%%%%%%%%%%%%%
% Mar 10, 2017
% Add weight, for recovering original pixels
%%%%%%%%%%%%%%%%%%%%%
% Mar 9, 2017
% ADMM + TV Prior Processing
%%%%%%%%%%%%%%%%%%%%%

function result = ProTV(I, rho, lambda, DRange, Bsize, sigma)

imageResolution = size(I);
% generating credibility map of each pixel, how reliable is this pixel
Mask1 = WeightGen(I, DRange, Bsize, sigma);     % Mask for Weight
Mask1r = reshape(Mask1, [prod(imageResolution),1]);
% Mask2 = ones(size(I));      % Mask for invalid pixels
% Mask2(b) = 0;
I(I<DRange(1)) = DRange(1)-2;   % if the pixel is invalid, give it a slightly lower value than minimum range


% Define functions
Extb = @(x) x(b);
% VProc = @(x) x.*Mask1;
% VtProc = @(x) x;
VFun = @(x) x.* Mask1r;
VtVFunAdd = @(x) VFun(x) + reshape(  rho*opDtx(opDx( reshape(x,imageResolution) )), [prod(imageResolution),1] );

x = ones(imageResolution)*mean(DRange);
x = reshape( x, [prod(imageResolution),1] );

Maxiter = 10;
u1 = zeros(imageResolution);
u2 = zeros(imageResolution);
z1 = zeros(imageResolution);
z2 = zeros(imageResolution);

% ADMM iteration
for Mi = 1:Maxiter
    % x update
    x = pcg(VtVFunAdd, reshape(I.*Mask1+rho*opDtx(cat(3,z1-u1,z2-u2)), [prod(imageResolution),1] ) , 1e-12, 500 );
    x = reshape( x, imageResolution );
    % making sure x is in the range
    x(x<DRange(1)) = DRange(1);
    x(x>DRange(2)) = DRange(2);
    % z update
    temp = opDx(x);
    x1 = temp(:,:,1);
    x2 = temp(:,:,2);
    z1 = zeros(imageResolution);
    z2 = zeros(imageResolution);
    D1 = x1 + u1;
    D2 = x2 + u2;
    for j=1:imageResolution(1)
        for k=1:imageResolution(2)
            if D1(j, k)>lambda/rho
                z1(j, k) = D1(j, k)-lambda/rho;
            elseif D1(j, k)<-lambda/rho
                z1(j, k) = D1(j, k)+lambda/rho;
            end
            if D2(j, k)>lambda/rho
                z2(j, k) = D2(j, k)-lambda/rho;
            elseif D2(j, k)<-lambda/rho
                z2(j, k) = D2(j, k)+lambda/rho;
            end
        end
    end
    % u update
    u1 = D1 - z1;
    u2 = D2 - z2;
    
% % Showing intermediate results    
%     figure;
%     imshow(x, [DRange(1), max(x(:))]);
%     colormap jet
%     colorbar
%     title('ADMM + TV');
%     print([pwd,'/',num2str(Mi)],'-dpng','-r0');
%     savefig([pwd,'/',num2str(Mi)]);
%     close;
    
end

result = x;

end