% clear all; close all;

%% Parameters
% brightness/strength of fog
fog_val = 1;
% if true, this makes fog nonexistent where shallow, equal where deep
HARD_CUTOFF = false;
% fog cutoff threshold if using hard cutoff
depth_thresh = 0.3;
% if true, this makes fog equal along x and y axis (i.e., no fog source)
FLAT_FOG = true;
% if true, then implementing bokeh effect and defining a focal plane. Otherwise it works for fog simualtion
flagbokeh = true; 
% point position at source of fog if not using flat fog
px = 471;
py = 1437;
% sigma for x and y gaussians - how quickly does fog spread
sx = 100000;
sy = 100000;

%% Preprocess
% read in clear image
fileread = true;
filename1 = 'Pic1\view0.png';
filename2 = 'Pic1\view2.png';
% filename1 = 'Pic4\im0.png';
% filename2 = 'Pic4\im1.png';
% filename1 = 'patio.jpg';
orig = im2double(imread(filename1));

if fileread  
    [depth, xmin] = Disparity_f(filename1, filename2);
    depth = depth(:,xmin:end);
else 
    load('depth_map.mat');
%     depth = im2double(imread('patio_depth.jpg')); xmin=1;
    depth = Ipf(:,xmin:end);
end
orig = orig(:,xmin:end,:);
% For paramters related to bokeh effect
if flagbokeh 
    centerdepth = 90;
    blurratio = 1;
    brightratio = 0;
else 
    centerdepth = max(depth(:))-.2*max(depth(:));
    blurratio = .3;
    brightratio = 0.7;
end

% naive method to fill in holes in depth map
for i = 1:size(depth,1)
    for j = 1:size(depth,2)
        if depth(i,j) < 0.02
            if i== 1 && j == 1
                depth(i,j) = 0.02;
            elseif i == 1
                depth(i,j) = depth(i,j-1);
            elseif j == 1
                depth(i,j) = depth(i-1,j);
            else
                depth(i,j) = (depth(i,j-1) + depth(i-1,j))/2;
            end
        end
    end
end

% create blurry image
if flagbokeh
    h = fspecial('gaussian',20,20);
else
    h = fspecial('gaussian',10,2);
end
blur = imfilter(orig,h);

% create brightness image
bright = fog_val*ones(size(orig));

%% Create Image
if HARD_CUTOFF
    % create flat foggy image
    fog = orig;
    for i = 1:size(fog,1)
        for j = 1:size(fog,2)
            if mod(i+j,2) == 0
                fog(i,j,:) = [fog_val fog_val fog_val];
            end
        end
    end
    fog = imfilter(fog,h);
    % create background/foreground mask
    mask = repmat((depth < depth_thresh),1,1,3);
    total = zeros(size(orig));
    % foreground - no fog
    total(~mask) = orig(~mask);
    % background - all equal fog
    total(mask) = fog(mask);
    
else
    if FLAT_FOG
        % fog equal along x and y axes
        Bx = ones(size(orig,1),size(orig,2));
        By = ones(size(orig,1),size(orig,2));
    else
        % grid to calculate weights
        x = 1:size(orig,1);
        y = 1:size(orig,2);
        [X,Y] = meshgrid(y,x);
        % weight on foggy image on x and y axes
        Bx = exp(-((X-px).^2)/sx);
        By = exp(-((Y-py).^2)/sy);
    end
    % transmittance map - fog decreases exponentially
    if flagbokeh
        t = abs(depth-centerdepth);
        t = t/max(t(:));
        t = 1 - (1-t).^(6);
    else
        t = exp(-2.3*fog_val/max(depth(:))*depth);
    end

    weight = repmat(t.*Bx.*By,1,1,3);
    
    total = orig.*(1-weight) + (blurratio*blur + brightratio*bright).*weight;
end

% figure;
% subplot(1,2,1);
% imshow(orig);
% title('Original');
% subplot(1,2,2);
% imshow(total);
% title('Depth Fog');



imwrite(total,'Result.png');
